import { each, get } from 'lodash';
/**
 * 撤销
 * @param state
 * @returns {(*&void&{undo: *, redo: *})|(*&T&{undo: *, redo: *})}
 */
export function undo(state) {
    const { undo, redo } = state;
    if (undo.length == 0)
        return state;
    const nextState = undo.pop();
    const prevState = {};
    each(nextState, (_, key) => (prevState[key] = get(state, key)));
    redo.push(prevState);
    return {
        ...state,
        ...nextState,
        undo,
        redo,
    };
}
/**
 * 重做
 * @param state
 * @returns {(*&void&{undo: *, redo: *})|(*&T&{undo: *, redo: *})}
 */
export function redo(state) {
    const { undo, redo } = state;
    if (redo.length == 0)
        return state;
    const nextState = redo.pop();
    const prevState = {};
    each(nextState, (_, key) => (prevState[key] = get(state, key)));
    undo.push(prevState);
    return {
        ...state,
        ...nextState,
        undo,
        redo,
    };
}
