import { NODE_PROPS_TYPES, PROPS_TYPES } from './ConfigTypes';
/**
 * 组件属性的配置信息
 */
export interface PropsConfigType {
    [propName: string]: PropInfoType;
}
/**
 * 单个属性的具体配置
 */
export interface PropInfoType {
    label?: string;
    type: PROPS_TYPES | PROPS_TYPES[];
    tip?: string;
    childPropsConfig?: PropsConfigType | PropsConfigType[];
    isAdd?: boolean;
    rules?: any[];
    placeholder?: string;
    enumData?: string[];
    defaultValue?: any;
    hasUnit?: boolean;
    isShowColor?: boolean;
    inputColProps?: any;
    maxTagCount?: number;
    min?: number;
    max?: number;
    stringCount?: number;
    formItemProps?: any;
    unDelete?: boolean;
    isRequired?: boolean;
}
/**
 * 节点属性配置信息，节点属性是这样定义的，一个属性的值可为html标签或者react组件的属性就是节点属性
 * react的子节点嵌套实际是将子节点作为属性children的值，所以children也可以作为属性节点配置
 */
export interface NodeProps {
    type: NODE_PROPS_TYPES.reactNode | NODE_PROPS_TYPES.functionReactNode;
    tip?: string;
    label?: string;
    childNodesRule?: string[];
    isRequired?: boolean;
    isOnlyNode?: boolean;
    params?: string[];
}
export interface NodePropsConfigType {
    [propName: string]: NodeProps;
}
/**
 * 弹窗类组件配置显示映射，目的是为了将弹窗类的组件展示约束到设计面板中去
 */
export interface MirrorModalFieldType {
    displayPropName: string;
    mounted?: {
        propName: string;
        type: PROPS_TYPES.function | PROPS_TYPES.string;
    };
    style?: any;
}
/**
 * 组件配置的数据结构
 */
export interface ComponentConfigTypes extends Omit<NodeProps, 'type' | 'tip' | 'params' | 'label'> {
    fatherNodesRule?: string[];
    nodePropsConfig?: NodePropsConfigType;
    mirrorModalField?: MirrorModalFieldType;
    propsConfig: PropsConfigType;
}
