import { each, get, isEmpty } from 'lodash';
import { LEGO_BRIDGE } from '../store';
/**
 * 根节点key
 */
export const ROOT = '0';
export const copyConfig = (handleInfo, selectedKey, newKey) => {
    const { componentConfigs, propsConfigSheet } = handleInfo;
    if (propsConfigSheet[selectedKey]) {
        propsConfigSheet[newKey] = propsConfigSheet[selectedKey];
    }
    const vDom = componentConfigs[selectedKey];
    const childNodes = vDom.childNodes;
    if (!childNodes) {
        componentConfigs[newKey] = vDom;
    }
    else {
        const newVDom = { ...vDom };
        componentConfigs[newKey] = newVDom;
        if (Array.isArray(childNodes)) {
            newVDom.childNodes = copyChildNodes(handleInfo, childNodes);
        }
        else {
            const newChildNodes = {};
            each(childNodes, (nodes, propName) => {
                newChildNodes[propName] = copyChildNodes(handleInfo, nodes);
            });
            newVDom.childNodes = newChildNodes;
        }
    }
};
function copyChildNodes(handleInfo, childNodes) {
    const { componentConfigs } = handleInfo;
    const newChildKeys = [];
    for (const oldKey of childNodes) {
        const newChildKey = getNewKey(componentConfigs);
        newChildKeys.push(`${newChildKey}`);
        copyConfig(handleInfo, oldKey, newChildKey);
    }
    return newChildKeys;
}
export function deleteChildNodes(handleInfo, childNodes, propName) {
    if (Array.isArray(childNodes)) {
        deleteArrChild(handleInfo, childNodes);
    }
    else {
        each(childNodes, (propChildNodes, key) => {
            if (propName) {
                if (key === propName) {
                    deleteArrChild(handleInfo, propChildNodes);
                }
            }
            else {
                deleteArrChild(handleInfo, propChildNodes);
            }
        });
    }
}
function deleteArrChild(handleInfo, childNodes) {
    for (const key of childNodes) {
        const childNodesInfo = handleInfo.componentConfigs[key].childNodes;
        if (childNodesInfo) {
            deleteChildNodes(handleInfo, childNodesInfo);
        }
        delete handleInfo.componentConfigs[key];
        delete handleInfo.propsConfigSheet[key];
    }
}
/**
 * 获取路径
 * */
export function getLocation(key, propName) {
    const basePath = [key, 'childNodes'];
    return propName ? [...basePath, propName] : basePath;
}
/**
 * 生成新的Key
 * @param dragVDOMAndPropsConfig
 * @param rootKey
 */
export const generateNewKey = (dragVDOMAndPropsConfig, rootKey) => {
    const { vDOMCollection, propsConfigCollection } = dragVDOMAndPropsConfig;
    const keyMap = {};
    const newVDOMCollection = {};
    let newPropsConfigCollection;
    let newKey = rootKey;
    each(vDOMCollection, (vDom, key) => {
        ++newKey;
        if (key === ROOT) {
            newKey = rootKey;
        }
        keyMap[key] = `${newKey}`;
        newVDOMCollection[newKey] = vDom;
    });
    each(newVDOMCollection, (vDom) => {
        const childNodes = vDom.childNodes;
        if (!childNodes)
            return;
        if (Array.isArray(childNodes)) {
            vDom.childNodes = childNodes.map((key) => keyMap[key]);
        }
        else {
            const newChildNodes = {};
            each(childNodes, (nodes, propName) => {
                newChildNodes[propName] = nodes.map((key) => keyMap[key]);
            });
            vDom.childNodes = newChildNodes;
        }
    });
    if (propsConfigCollection) {
        newPropsConfigCollection = {};
        each(propsConfigCollection, (v, k) => {
            newPropsConfigCollection[keyMap[k]] = v;
        });
    }
    return { newVDOMCollection, newPropsConfigCollection };
};
/**
 * 获取字段在props中的位置
 * @param fieldConfigLocation
 * @returns {string}
 */
export const getFieldInPropsLocation = (fieldConfigLocation) => {
    return fieldConfigLocation
        .split('.')
        .filter((location) => location !== 'childPropsConfig');
};
/**
 * 处理子节点非空
 * @param selectedInfo
 * @param componentConfigs
 * @param payload
 */
export const handleRequiredHasChild = (selectedInfo, componentConfigs) => {
    const { selectedKey, propName: selectedPropName } = selectedInfo;
    const { componentName, childNodes } = componentConfigs[selectedKey];
    const { nodePropsConfig, isRequired } = getComponentConfig(componentName);
    return ((selectedPropName &&
        nodePropsConfig[selectedPropName].isRequired &&
        !get(childNodes, selectedPropName)) ||
        (isRequired && !childNodes));
};
export function shallowEqual(objA, objB) {
    for (const k of Object.keys(objA)) {
        if (objA[k] !== objB[k])
            return false;
    }
    return true;
}
export function deleteChildNodesKey(childNodes, deleteKey, parentPropName) {
    if (Array.isArray(childNodes)) {
        const resultChildNodes = childNodes.filter((nodeKey) => nodeKey !== deleteKey);
        return restObject(resultChildNodes);
    }
    else {
        const resultChildNodes = childNodes[parentPropName].filter((nodeKey) => nodeKey !== deleteKey);
        if (resultChildNodes.length === 0) {
            delete childNodes[parentPropName];
        }
        else {
            childNodes[parentPropName] = resultChildNodes;
        }
        return restObject(childNodes);
    }
}
export function getComponentConfig(componentName) {
    const allComponentConfigs = get(LEGO_BRIDGE, [
        'config',
        'AllComponentConfigs',
    ]);
    if (!allComponentConfigs) {
        error('Component configuration information set not found！! !');
    }
    const componentConfig = allComponentConfigs[componentName];
    if (componentName && !componentConfig) {
        warn(`${componentName} configuration information not found！! !`);
    }
    return componentConfig;
}
export function isContainer(componentName) {
    const containers = get(LEGO_BRIDGE, ['config', 'containers']);
    if (!containers) {
        error(`Please configure the container category of the components`);
    }
    else {
        return containers.includes(componentName);
    }
}
export function error(msg) {
    console.error(msg);
    throw new Error(msg);
}
export function warn(msg) {
    const warn = get(LEGO_BRIDGE, ['config', 'warn']);
    if (warn) {
        warn(msg);
    }
    else {
        console.warn(msg);
    }
    return true;
}
export function getNewKey(componentConfigs) {
    const lastKey = Object.keys(componentConfigs).pop();
    return Number(lastKey) + 1;
}
export function restObject(obj, field) {
    if (field) {
        delete obj[field];
    }
    if (Object.keys(obj).length === 0)
        return undefined;
    return obj;
}
export function handleRules(componentConfigs, dragKey, selectedKey, propName, isForbid) {
    /**
     * 获取当前拖拽组件的父组件约束，以及属性节点配置信息
     */
    const dragComponentName = componentConfigs[dragKey].componentName;
    const dropComponentName = componentConfigs[selectedKey].componentName;
    const { fatherNodesRule } = getComponentConfig(dragComponentName);
    const { nodePropsConfig, childNodesRule } = getComponentConfig(dropComponentName);
    /**
     * 子组件约束限制，减少不必要的组件错误嵌套
     */
    const childRules = propName
        ? nodePropsConfig[propName].childNodesRule
        : childNodesRule;
    if (childRules && !childRules.includes(dragComponentName)) {
        !isForbid &&
            warn(`${propName || dropComponentName}:only allow drag and drop to add${childRules.toString()}`);
        return true;
    }
    /**
     * 父组件约束限制，减少不必要的组件错误嵌套
     */
    if (fatherNodesRule &&
        !fatherNodesRule.includes(propName ? `${dropComponentName}.${propName}` : `${dropComponentName}`)) {
        !isForbid &&
            warn(`${dragComponentName}:Only allowed as a child node or attribute node of${fatherNodesRule.toString()}`);
        return true;
    }
    return false;
}
export function combineReducers(brickReducer, customReducer) {
    return (state, action) => {
        const newState = brickReducer(state, action);
        if (customReducer) {
            return customReducer(newState, action);
        }
        else {
            return newState;
        }
    };
}
export function createTemplateConfigs(templateConfigs, templatePropsConfigSheet, componentConfigs, propsConfigSheet, childNodes) {
    if (Array.isArray(childNodes)) {
        for (const key of childNodes) {
            templateConfigs[key] = componentConfigs[key];
            if (propsConfigSheet[key]) {
                templatePropsConfigSheet[key] = propsConfigSheet[key];
            }
            const { childNodes } = templateConfigs[key];
            if (childNodes)
                createTemplateConfigs(templateConfigs, templatePropsConfigSheet, componentConfigs, propsConfigSheet, childNodes);
        }
    }
    else {
        each(childNodes, (childKeys) => {
            if (!isEmpty(childKeys))
                createTemplateConfigs(templateConfigs, templatePropsConfigSheet, componentConfigs, propsConfigSheet, childKeys);
        });
    }
}
export function createTemplate(state) {
    const { selectedInfo, componentConfigs, propsConfigSheet } = state;
    const { selectedKey } = selectedInfo;
    const templateConfigs = { [ROOT]: componentConfigs[selectedKey] };
    const templatePropsConfigSheet = { [ROOT]: propsConfigSheet[selectedKey] };
    const { childNodes } = componentConfigs[selectedKey];
    if (!isEmpty(childNodes)) {
        createTemplateConfigs(templateConfigs, templatePropsConfigSheet, componentConfigs, propsConfigSheet, childNodes);
    }
    return { templateConfigs, templatePropsConfigSheet };
}
